/*
 * ProcessorFactory.java
 */

package original;

import java.lang.reflect.*;
import java.util.Properties;
import util.LoaderUtils;

public class ProcessorFactory
{
    private static ProcessorFactory instance = new ProcessorFactory();

    private Properties props;

    protected ProcessorFactory() {
        props = LoaderUtils.loadProperties("processor.properties");
    }

    public static ProcessorFactory getInstance() {
        return instance;
    }

    public Processor getProcessor(String name) {
        String key = "processor.class." + name;
        String className = props.getProperty(key);
        if (className == null || className.length() == 0) {
            return new original.impl.ProcessorImpl();
        }
        try {
            Class clazz = LoaderUtils.loadClass(className);
            return createProcessor(clazz, name);
        } catch (RuntimeException e) {
            throw e;
        } catch (Exception e) {
            throw new SystemException(e);
        }
    }

    static final Class[] classArgs = { String.class };

    protected Processor createProcessor(Class clazz, String name)
            throws Exception {
        try {
            // ȂRXgN^̌ĂяoB
            return (Processor)clazz.newInstance();
        } catch (Exception e) {}

        try {
            // RXgN^̌ĂяoB
            Constructor c = clazz.getConstructor(classArgs);
            return (Processor)c.newInstance(new Object[] { name });
        } catch (Exception e) {}

        try {
            // NX\bh create ̌ĂяoB
            Method m = clazz.getMethod("create", classArgs);
            return (Processor)m.invoke(null, new Object[] { name });
        } catch (InvocationTargetException e) {
            Throwable t = e.getTargetException();
            if (t instanceof Error) {
                throw (Error)t;
            } else if (t instanceof RuntimeException) {
                throw (RuntimeException)t;
            } else {
                throw new SystemException(t);
            }
        }
    }
}
